'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _babelPluginSyntaxFlow = require('babel-plugin-syntax-flow');

var _babelPluginSyntaxFlow2 = _interopRequireDefault(_babelPluginSyntaxFlow);

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

var _babelTemplate = require('babel-template');

var _babelTemplate2 = _interopRequireDefault(_babelTemplate);

var _lodash = require('lodash.snakecase');

var _lodash2 = _interopRequireDefault(_lodash);

var _lodash3 = require('lodash.camelcase');

var _lodash4 = _interopRequireDefault(_lodash3);

var _babelAddFlowComments = require('babel-add-flow-comments');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

// import blog from 'babel-log'

var constantCase = function constantCase(str) {
  return (0, _lodash2.default)(str).toUpperCase();
};

var babylonOpts = { sourceType: 'module', plugins: ['flow'] };

var wrapTemp = function wrapTemp(tmpl) {
  return (0, _babelTemplate2.default)(tmpl, babylonOpts);
};

var builders = {
  actionCreater: wrapTemp('function NAME(PARAMS): TYPE {\n    return VALUE;\n  }')
};

function createActionCreater(name, props, params) {
  var typeIdentifier = t.identifier('type');
  return builders.actionCreater({
    NAME: t.identifier((0, _lodash4.default)(name)),
    TYPE: t.identifier(name),
    VALUE: t.objectExpression([t.objectProperty(typeIdentifier, t.identifier(constantCase(name)))].concat(_toConsumableArray(props))),
    PARAMS: params
  });
}

exports.default = function () {
  return {
    inherits: _babelPluginSyntaxFlow2.default,
    name: 's2s-action-creater',
    visitor: {
      Program: {
        exit: function exit(programPath, state) {
          var file = state.file;

          var basename = file.opts.basename;
          (0, _babelAddFlowComments.removeFlowComment)(file.ast.comments);

          var imports = [];
          var typeNames = [];
          var actions = [];
          var funcs = [];

          programPath.traverse({
            ImportDeclaration: function ImportDeclaration(path) {
              imports.push(path.node);
            },
            VariableDeclarator: function VariableDeclarator(path) {
              var name = path.get('id').node.name;
              if (name !== 'Actions') {
                actions.push(name);
              }
            },
            TypeAlias: function TypeAlias(path) {
              var name = path.get('id').node.name;

              if (name === 'Action') {
                return;
              }
              typeNames.push(name);

              var params = [];
              var props = [];

              var _iteratorNormalCompletion = true;
              var _didIteratorError = false;
              var _iteratorError = undefined;

              try {
                for (var _iterator = path.get('right').node.properties[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                  var _ref = _step.value;
                  var _key = _ref.key;
                  var value = _ref.value;

                  if (_key.name !== 'type') {
                    var p = t.identifier(_key.name);

                    props.push(t.objectProperty(p, p, false, true));

                    p.typeAnnotation = t.typeAnnotation(value);
                    params.push(p);
                  }
                }
              } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
              } finally {
                try {
                  if (!_iteratorNormalCompletion && _iterator.return) {
                    _iterator.return();
                  }
                } finally {
                  if (_didIteratorError) {
                    throw _iteratorError;
                  }
                }
              }

              funcs.push(createActionCreater(name, props, params));
            }
          });

          function createImport(arr) {
            var specifiers = [];
            var _iteratorNormalCompletion2 = true;
            var _didIteratorError2 = false;
            var _iteratorError2 = undefined;

            try {
              for (var _iterator2 = arr[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                var x = _step2.value;

                var i = t.identifier(x);
                specifiers.push(t.importSpecifier(i, i));
              }
            } catch (err) {
              _didIteratorError2 = true;
              _iteratorError2 = err;
            } finally {
              try {
                if (!_iteratorNormalCompletion2 && _iterator2.return) {
                  _iterator2.return();
                }
              } finally {
                if (_didIteratorError2) {
                  throw _iteratorError2;
                }
              }
            }

            return t.importDeclaration(specifiers, t.stringLiteral('./' + basename));
          }

          var constImport = createImport(actions);
          var typeImport = createImport(typeNames);

          // $FlowFixMe
          typeImport.importKind = 'type';

          programPath.node.body = [].concat(imports, [constImport, typeImport, t.noop()], funcs);

          (0, _babelAddFlowComments.addFlowComment)(programPath);
        }
      }
    }
  };
};